/* Real-Time audio broadcast station - SERVER.
 * Version 0.0.2a -> July 3, 1996.
 * Written by Electronic One.  <dkaye@cyberspc.mb.ca>
 */

#include <stdio.h>
#include <syslog.h>
#include "curses.h"
#include <signal.h>
#include <stdlib.h>

/* Custom includes */
#include "errorcodes.h"
#include "version.h"
#include "char2short.h"

/* The ultra-elite audio compressor */
#include "gsm.h"

/* Global for the benefit of cleanup() */
FILE *inf, *outf=stdout;
int errcode=0;
gsm handle;

static void cleanup(void);

int main(int argc, char*argv[])
{
   int i, c, value=0;

   unsigned char *buf=(unsigned char*)malloc(33);
   short *sample=(signed short*)malloc(160*2);

   setvbuf(outf, NULL, _IOFBF, 65536);
   
   initscr();
   cbreak();  /* This is the call to make the terminal go into char mode */
   
   openlog("RTAudio.Server", LOG_PID, LOG_DAEMON);
   syslog(LOG_INFO, "Start v%s -> %s.", RTA_VERSION, RTA_DATE);

/*   
   signal(SIGTERM, cleanup);
   signal(SIGQUIT, cleanup);
   signal(SIGINT, cleanup);  
   signal(SIGABRT, cleanup);
   signal(SIGTSTP, cleanup);
   signal(SIGIO, cleanup);
*/
   
   atexit(cleanup);
   
   if((!errcode) && ((inf = fopen("/dev/audio", "rb")) == NULL))
     {
	printf("Sorry, someone is already connected.\n");
	syslog(LOG_NOTICE, "Multiple connection attempt");
	errcode = ERR_OPEN;
     }

   if((!errcode) && (!(handle = gsm_create())))
     {
	syslog(LOG_ERR, "Couldn't create GSM handle");
	errcode = ERR_GSM_CREATE;
     }
   
   if((!errcode) && gsm_option(handle, GSM_OPT_FAST, &value))
     {
	/* Note to hackers: Don't enable it.  It just causes a FPE. */
	syslog(LOG_ERR, "Couldn't disable fast compression option");
	errcode = ERR_OPTION;
     }

   while(!errcode)
     {
        for (i = 0; (!errcode) && (i < 160) && ((c = fgetc(inf)) != EOF); i++) 
          sample[i] = U2S(c);

        if ((!errcode) && ((c == EOF) || ferror(inf)))
          {
	     syslog(LOG_ERR, "Error reading from sound card");
	     errcode = ERR_READ;
	  }

	gsm_encode(handle, sample, buf);
	
	if((!errcode) && (fwrite((char*)buf, sizeof(*buf), 33, outf) != 33))
	  {
	     syslog(LOG_NOTICE, "Error writing to output");
	     errcode = ERR_WRITE;
	  }
     }

   gsm_destroy(handle);
   fflush(outf);
   setvbuf(outf, NULL, _IOLBF, 1024);
   fclose(inf);
   nocbreak();
   syslog(LOG_NOTICE, "Terminated by signal %d, errcode=%d", 0, errcode);
   closelog();
  
   return errcode;
}

static void cleanup(void)
{
   gsm_destroy(handle);
   fflush(outf);
   setvbuf(outf, NULL, _IOLBF, 1024);
   fclose(inf);
   nocbreak();
   openlog("RTAudio.Server", LOG_PID, LOG_DAEMON);
   syslog(LOG_NOTICE, "Terminated with errcode=%d", errcode);
   closelog();
}



